#!/usr/bin/env python

"""merge_annotations.py: Merge the annotations in a provided directory and print out some basic statistics.
   This script is intended to be a simple example of using the agreement annotations that can be improved per user need.
   usage: python merge_annotations.py <directory>

   sample output:

   There are 1866 unique annotations
   There are 473 livejournal annotations and 1393 wikipedia annotations
   609 agree 554 disagree 703 none
   livejournal: 390 agree 83 disagree 0 none
   wikipedia: 219 agree 471 disagree 703 none

   NOTE: can also call csv() to print annotations in csv format

"""

__author__      = "Sara Rosenthal"
__version__     = "1.0.0"
__email__       = "sara@cs.columbia.edu"

import json, sys, os

class Agreement:
    quoteid=-1
    responseid=-1
    quote=""
    response=""
    agreement_type="none"
    agreement_mode="none"
    annotator = ""
    source = ""
    id = ""
    
    def __init__(self, qId, rId, q, r, t, m,s,a,id):
        self.quoteid = qId
        self.responseid = rId
        self.quote = q
        self.response = r
        self.agreement_type = t
        self.agreement_mode = m
        self.annotator = a
        if "livejournal" in s:
            self.source = "livejournal"
            self.id = s
        else:
            self.source = "wikipedia"
            self.id = id
    def toString(self):
        return self.agreement_type,"(", self.agreement_mode, "): ", self.quoteid, self.quote, self.responseid, self.response, " by annotator: " , self.annotator
    def toCSV(self):
            print self.agreement_type ,",", self.agreement_mode, ", ", self.quoteid, ", ", self.quote, ", ", self.responseid, ", ", self.response, ", " , self.annotator
    
    
# read file and load annotations
def load(file,agreements):
    f = open(file,'r')

    #agreements = {};

    for line in f:
        data = line.split(";",3)
        #print data[-1]
        ann = json.loads(data[-1])
        for a in ann:
            key = "id" + a['targetId']+a['subjectId']+''.join(a['target'])+''.join(a['subject'])
            a = Agreement(a['targetId'],a['subjectId'],''.join(a['target']),''.join(a['subject']),a['type'],a['mode'],data[0],data[1],data[2])
            # remove duplicates and disagreements in annotations (this can be improved to take majority)
            if key in agreements:
                # print the disagreement
                #print agreements[key].toString() , "<---->" , a.toString()
                if agreements[key] == -1 or agreements[key].agreement_type == a.agreement_type:
                    continue
                agreements[key] = -1
            else:
                agreements[key] = a
    f.close()

def count (agreements, type="", source=""):
    i = 0;
    for key in agreements:
        if agreements[key] == -1:
            continue;
        if (agreements[key].agreement_type == type or type == "") and (agreements[key].source == source or source == ""):
            i = i+1
    return i

def stats(agreements):
    print "There are" , count(agreements) , "unique annotations"
    print "There are" , count(agreements,"","livejournal") , "livejournal annotations and" , count(agreements,"","wikipedia") , "wikipedia annotations" 
    print count(agreements,"agreement") , "agree" ,  count(agreements,"disagreement") , "disagree" ,  count(agreements,"none") , "none"
    print "livejournal:" , count(agreements,"agreement","livejournal") , "agree" ,  count(agreements,"disagreement","livejournal") , "disagree" ,  count(agreements,"none","livejournal") , "none"
    print "wikipedia:" , count(agreements,"agreement","wikipedia") , "agree" ,  count(agreements,"disagreement","wikipedia") , "disagree" ,  count(agreements,"none","wikipedia") , "none"

def csv(agreements):
    for key in agreements:
        if agreements[key] == -1:
            continue
        agreements[key].toCSV()
 
def main (argv):
    # get list of files to read in
    files = os.listdir(argv[0])

    agreements = {}
    for f in files:
        if not f.endswith("txt"):
            continue
        # load all the agreements
        load(argv[0]+"/"+f,agreements)
    # print stats
    stats(agreements)
    # print in csv format
    #csv(agreements)
    
main(sys.argv[1:])
